//==============================================================================
// TITLE: COpcHdaItem.h
//
// CONTENTS:
// 
// A collection of historical values for an item.
//
// (c) Copyright 2003-2004 The OPC Foundation
// ALL RIGHTS RESERVED.
//
// DISCLAIMER:
//  This code is provided by the OPC Foundation solely to assist in 
//  understanding and use of the appropriate OPC Specification(s) and may be 
//  used as set forth in the License Grant section of the OPC Specification.
//  This code is provided as-is and without warranty or support of any sort
//  and is subject to the Warranty and Liability Disclaimers which appear
//  in the printed OPC Specification.
//
// MODIFICATION LOG:
//
// Date       By    Notes
// ---------- ---   -----
// 2003/12/17 RSA   Initial implementation.

#ifndef _COpcHdaItem_H_
#define _COpcHdaItem_H_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "COpcHdaAttribute.h"
#include "COpcHdaItemValue.h"

//============================================================================
// MACRO:   OPCHDA_MAX_UPDATE_INTERVAL
// PURPOSE: Defines the maximum number of values per item that can be returned in a single call.

#define OPCHDA_MAX_UPDATE_INTERVAL 10000000

//==============================================================================
// CLASS:   COpcHdaItem
// PURPOSE: A source server for data exchange data.

class COpcHdaHistorian;

class COpcHdaItem : public IOpcXmlSerialize
{
    OPC_CLASS_NEW_DELETE()

public:

    //==========================================================================
    // Operators

    // Constructor
    COpcHdaItem(const COpcString& cItemID,void *userObj,COpcHdaHistorian *o) { Init(); m_cItemID = cItemID; userObject=userObj; owner=o; }

    // Destructor 
    ~COpcHdaItem() { Clear(); }
    
    //==========================================================================
    // Public Methods

    // GetItemID
    COpcString GetItemID() { return m_cItemID; }

	// BuildAddressSpace
	bool BuildAddressSpace();

	// ClearAddressSpace
	void ClearAddressSpace();

	// returns the current value of an attribute.
	bool GetAttribute(DWORD dwID, OpcXml::AnyType& cValue);

	// checks whether the item meets the criteria specified by the filter.
	bool Match(COpcHdaBrowseFilter& cFilter);

	// saves the data for the item.
	void SaveData();

	// reads and modified values within the specified interval.
	HRESULT ReadModified(
		LONGLONG             llStartTime,
		LONGLONG             llEndTime,
		UINT                 uNumValues,
		UINT                 uStartIndex,
		OPCHDA_MODIFIEDITEM& cItem
	);

	// reads item attributes over the specified interval.
	HRESULT ReadAttribute(
		LONGLONG          llStartTime,
		LONGLONG          llEndTime,
		DWORD             dwAttributeID, 
		OPCHDA_ATTRIBUTE& cAttribute
	);

	// reads the item annotations over the specified interval.
	HRESULT ReadAnnotations(
		LONGLONG           llStartTime,
		LONGLONG           llEndTime,
		OPCHDA_ANNOTATION& cAnnotation
	);

	// inserts the item annoations at the specified timestamp.
	HRESULT InsertAnnotation(
		LONGLONG           llTimestamp,
		OPCHDA_ANNOTATION& cAnnotation
	);

	// updates the item values.
	HRESULT Update(
		OPCHDA_EDITTYPE eEditType,
		LONGLONG        llTimestamp,
		VARIANT&        vValue,
		DWORD           dwQuality
	);
	
	// deletes the values within the specified interval.
	HRESULT Delete(
		LONGLONG llStartTime,
		LONGLONG llEndTime
	);

    //==========================================================================
    // IOpcXmlSerialize

    // Init
    virtual void Init();

    // Clear
    virtual void Clear();

    // Read
    virtual bool Read(COpcXmlElement& cElement);

    // Write
    virtual bool Write(COpcXmlElement& cElement);

	void *getUserObject() { return userObject; };

private:
    
	//==========================================================================
    // Private Methods

	HRESULT GetCurrentValue(OPCHDA_ATTRIBUTE& cAttribute);

	// adds a new record to the modified value list.
	void CreateModifiedValue(OPCHDA_EDITTYPE eEditType, COpcHdaItemValue& cValue);

	COpcHdaHistorian *owner;

	// identity attributes.
    COpcString m_cItemID;	
    COpcString m_cItemName;	
	COpcString m_cDescription;
	COpcString m_cEngUnits;
	COpcString m_cDeriveEquation;
	COpcString m_cNodeName;
	COpcString m_cProcessName;
	
	// data description attributes.
//	VARTYPE    m_vtDataType;
	double     m_dblNormalMax;
	double     m_dblNormalMin;
	LONGLONG   m_llMaxInterval;
	LONGLONG   m_llMinInterval;
	double     m_dblDeviation;
	short      m_eDeviationType;
	double     m_dblLowEntryLimit;
	double     m_dblHighEntryLimit;


	// all other attributes.
	COpcMap<DWORD,OpcXml::AnyType*> m_cAttributes;

	// actual data.
	bool                       m_bLoaded;

	COpcHdaModifiedValueArray  m_cModifiedValues;
	COpcHdaAnnotationArray     m_cAnnotations;

	void *userObject;

	friend class COpcHdaItemValues;
};

//============================================================================
// TYPEDEF: COpcHdaItemMap
// PURPOSE: A table of device items indexed by item id.

typedef COpcMap<COpcString,COpcHdaItem*> COpcHdaItemMap;

#endif // _COpcHdaItem_H_
