//==============================================================================
// TITLE: COpcHdaItemValues.cpp
//
// CONTENTS:
// 

#include "StdAfx.h"
#include "COpcHdaItemValues.h"
#include "COpcHdaAttribute.h"
#include "COpcHdaHistorian.h"
#include "COpcHdaTime.h"

//============================================================================
// Local Declarations

#define TAG_NAME       _T("Name")
#define TAG_START_TIME _T("StartTime")
#define TAG_TIMESTAMP  _T("Timestamp")

#define TICKS_PER_SECOND 10000000
/*
COpcHdaItemValues *GetValuesForItem(COpcHdaItem *item,LONGLONG llStartTime,LONGLONG llEndTime){
	COpcHdaItemValues *res=new COpcHdaItemValues(item,25);

	for(LONGLONG i=0;i<25;i++){
		COpcVariant var=100.0*i;;
	
		res->AppendValue(var,llStartTime+(llEndTime-llStartTime)*i/25,OPCHDA_RAW);
	}

	return res;
}
*/
//============================================================================
// COpcHdaItemValues

//========================================================================
// IOpcXmlSerialize

void COpcHdaItemValues::AppendValue(COpcVariant &value,LONGLONG when, DWORD dwQuality){
	COpcHdaItemValue* pValue = new COpcHdaItemValue();
	pValue->dwQuality=dwQuality;
	pValue->llTimestamp=when;
	pValue->value=value;
	m_cValues.Append(pValue->llTimestamp, *pValue);

}

COpcHdaItemValues::COpcHdaItemValues(COpcHdaItem *it,int count){
	Init();

	item=it;
	m_cValues.SetCapacity(count);
}


// Init
void COpcHdaItemValues::Init()
{
	m_dblNormalMax      = 1000;
	m_dblNormalMin      = 0;
	m_llMaxInterval     = 10000000;
	m_llMinInterval     = 10000000;
	m_dblDeviation      = 0;
	m_eDeviationType    = 0;
	m_dblLowEntryLimit  = DBL_MIN;
	m_dblHighEntryLimit = DBL_MAX;

	m_llStartTime       = 0;
	m_llEndTime         = _I64_MAX;
	m_uFirstUseable     = -1;
	m_uLastUseable      = -1;

	m_bStepped          = false;
	m_bArchiving        = false;
	m_uWaveform         = 1;
	m_dblPeriod         = 30;
	
}
         
// Clear
void COpcHdaItemValues::Clear()
{
	m_cValues.RemoveAll();

    Init();
}

// Read
bool COpcHdaItemValues::Read(COpcXmlElement& cElement)
{
    Clear();

	return true;
}

// Write
bool COpcHdaItemValues::Write(COpcXmlElement& cElement)
{
	OPC_ASSERT(false);  
	
	// not implemented.
	
	return false;
}

// GetTimestampAtIndex
LONGLONG COpcHdaItemValues::GetTimestampAtIndex(UINT uIndex)
{
	if (m_cValues.IsValidIndex(uIndex))
	{
		return m_cValues[uIndex].llTimestamp;
	}

	return 0;
}

// ReadIndicies
bool COpcHdaItemValues::ReadIndicies(
	LONGLONG llStartTime,
	LONGLONG llEndTime,
	UINT     uMaxValues,
	bool     bIncludeBounds,
	UINT*    pIndicies,
	UINT&    uCount
)
{
	uCount = 0;

	// exactly one value requested.
	if (llStartTime == llEndTime)
	{
		// get the start index.
		UINT uIndex = m_cValues.FindIndex(llStartTime);

		// add single value.
		if (uIndex != -1)
		{
			pIndicies[uCount++] = uIndex;
		}

		return true;
	}

	// ascending series requested.
	else if (llEndTime == 0 || (llStartTime != 0 && llStartTime < llEndTime))
	{
		// get the start index.
		UINT uIndex = m_cValues.FindIndex(llStartTime);

		if (uIndex == -1)
		{			
			if (bIncludeBounds && llStartTime != 0)
			{
				pIndicies[uCount++] = m_cValues.FindIndexBefore(llStartTime);

				// check if space for data.
				if (uCount >= uMaxValues)
				{
					return false;
				}
			}
            
			uIndex = m_cValues.FindIndexAfter(llStartTime);
		}

		// check if no data exists after start time and before end time.
		if (uIndex == -1 || (llEndTime != 0  && llEndTime <=  GetTimestampAtIndex(uIndex)))
		{
			// no data to return.
			if (!bIncludeBounds)
			{
				return true;
			}

			// check if space for end bound. 
			if (uCount >= uMaxValues)
			{
				return false;
			}

			// add end bound.
			pIndicies[uCount++] = uIndex;
			return true;
		}

		// add start value to list.
		pIndicies[uCount++] = uIndex++;
		
		LONGLONG llTimestamp = 0;

		while (uCount < uMaxValues)
		{						
			// get timestamp of next sample.
			llTimestamp = GetTimestampAtIndex(uIndex);

			// check if no more data.
			if (llTimestamp == 0)
			{
				break;
			}

			// check for end of time range.
			if (llEndTime != 0 && llTimestamp >= llEndTime)
			{
				break;
			}
			
			// save index of sample an increment index.
			pIndicies[uCount++] = uIndex++;
		}

		// all data within the range found. add end bound before return.
		if (uCount < uMaxValues)
		{
			if (bIncludeBounds && llEndTime != 0)
			{
				pIndicies[uCount++] = (llTimestamp != 0)?uIndex:-1;
			}

			return true;
		}

		// all requested data returned.
		if (llEndTime == 0)
		{
			return true;
		}

		// check if next sample is the bound and bounds are not required.
		if (!bIncludeBounds)
		{
			if (llEndTime == 0 || llEndTime <=  GetTimestampAtIndex(uIndex))
			{
				return true;
			}
		}
		
		// not all data could be returned.
		return (llEndTime == 0);
	}

	// descending series requested.
	else if (llStartTime == 0 || llEndTime < llStartTime)
	{
		// swap end time and start time if start time is zero.
		if (llStartTime == 0)
		{
			llStartTime = llEndTime;
			llEndTime   = 0;
		}

		// get the start index.
		UINT uIndex = m_cValues.FindIndex(llStartTime);

		if (uIndex == -1)
		{			
			if (bIncludeBounds)
			{
				pIndicies[uCount++] = m_cValues.FindIndexAfter(llStartTime);

				// check if space for data.
				if (uCount >= uMaxValues)
				{
					return false;
				}
			}
            
			uIndex = m_cValues.FindIndexBefore(llStartTime);
		}

		// check if no data exists before start time and after end time.
		if (uIndex == -1 || (llEndTime != 0  && llEndTime >=  GetTimestampAtIndex(uIndex)))
		{
			// no data to return.
			if (!bIncludeBounds)
			{
				return true;
			}

			// check if space for end bound. 
			if (uCount >= uMaxValues)
			{
				return false;
			}

			// add end bound.
			pIndicies[uCount++] = uIndex;
			return true;
		}

		// add start value to list.
		pIndicies[uCount++] = uIndex--;
		
		LONGLONG llTimestamp = 0;

		while (uCount < uMaxValues)
		{						
			// get timestamp of next sample.
			llTimestamp =  GetTimestampAtIndex(uIndex);

			// check if no more data.
			if (llTimestamp == 0)
			{
				break;
			}

			// check for end of time range.
			if (llEndTime != 0  && llEndTime >=  GetTimestampAtIndex(uIndex))
			{
				break;
			}
			
			// save index of sample an increment index.
			pIndicies[uCount++] = uIndex--;
		}

		// all data within the range found. add end bound before return.
		if (uCount < uMaxValues)
		{
			if (bIncludeBounds && llEndTime != 0)
			{
				pIndicies[uCount++] = (llTimestamp != 0)?uIndex:-1;
			}

			return true;
		}

		// check if next sample is the bound and bounds are not required.
		if (!bIncludeBounds)
		{
			if (llEndTime == 0 || llEndTime >=  GetTimestampAtIndex(uIndex))
			{
				return true;
			}
		}
		
		// not all data could be returned.
		return (llEndTime == 0);
	}

	// no data to return;
	return true;
}

// ReadRaw
HRESULT COpcHdaItemValues::ReadRaw(
	LONGLONG     llStartTime,
	LONGLONG     llEndTime,
	UINT         uNumValues,
	bool         bIncludeBounds,
	UINT         uStartIndex,
	OPCHDA_ITEM& cItem
)
{
	SYSTEMTIME stStartTime;
	FILETIME   ftStartTime = ::OpcHdaFILETIMEFromInt64(llStartTime);
	FileTimeToSystemTime(&ftStartTime, &stStartTime);

	SYSTEMTIME stEndTime;
	FILETIME   ftEndTime = ::OpcHdaFILETIMEFromInt64(llEndTime);
	FileTimeToSystemTime(&ftEndTime, &stEndTime);

	SYSTEMTIME m_stStartTime;
	FILETIME   m_ftStartTime = ::OpcHdaFILETIMEFromInt64(m_llStartTime);
	FileTimeToSystemTime(&m_ftStartTime, &m_stStartTime);

	SYSTEMTIME m_stEndTime;
	FILETIME   m_ftEndTime = ::OpcHdaFILETIMEFromInt64(m_llEndTime);
	FileTimeToSystemTime(&m_ftEndTime, &m_stEndTime);

	// initialize empty item object.
	cItem.dwCount       = 0;
	cItem.haAggregate   = OPCHDA_NOAGGREGATE;
	cItem.hClient       = NULL;
	cItem.pvDataValues  = NULL;
	cItem.pdwQualities  = NULL;
	cItem.pftTimeStamps = NULL;

	// calculate the maximum number of values that could be returned.
	UINT uMaxValues = uNumValues + uStartIndex;

	if (uMaxValues == 0)
	{
		LONGLONG llUpperBound = (llEndTime >= llStartTime)?llEndTime:llStartTime;
		LONGLONG llLowerBound = (llEndTime <= llStartTime)?llEndTime:llStartTime;

		if (llUpperBound > m_llEndTime)	  { llUpperBound = m_llEndTime;   }
		if (llUpperBound < m_llStartTime) {	llUpperBound = m_llStartTime; }
		if (llLowerBound > m_llEndTime)   { llLowerBound = m_llEndTime;   }
		if (llLowerBound < m_llStartTime) {	llLowerBound = m_llStartTime; }

		// find the number of values in the interval.
		UINT uLower = 0;
		UINT uBound = 0;
		UINT uUpper = 0;

		m_cValues.FindBounds(llLowerBound, uLower, uBound);
		m_cValues.FindBounds(llUpperBound, uBound, uUpper);

		if (uUpper == -1)
		{
			uUpper = m_cValues.GetCount()-1;
		}

		if (uLower == -1)
		{
			uLower = 0;
		}

		// minimum is three values: one value plus two bounds.
		uMaxValues = uUpper - uLower + 3;
	}

	// apply any server defined limit on the number of values that can be returned.
	if (OPCHDA_MAX_VALUES > 0 && uMaxValues > OPCHDA_MAX_VALUES)
	{
		uMaxValues = OPCHDA_MAX_VALUES;
	}

	// lookup indicies of values within the time range.
	UINT* pIndicies = OpcArrayAlloc(UINT, uMaxValues);
	memset(pIndicies, 0, sizeof(UINT)*uMaxValues);

	UINT uCount = 0;
	
	bool bNoMoreData = ReadIndicies(
		llStartTime,
		llEndTime,
		uMaxValues,
		bIncludeBounds,
		pIndicies,
		uCount
	);

	// no data found.
	if (uCount == 0)
	{
		OpcFree(pIndicies);
		return OPC_S_NODATA;
	}

	cItem.dwCount = uCount-uStartIndex;

	// allocate arrays.
	cItem.pvDataValues  = OpcArrayAlloc(VARIANT, cItem.dwCount);
	cItem.pdwQualities  = OpcArrayAlloc(DWORD, cItem.dwCount);
	cItem.pftTimeStamps = OpcArrayAlloc(FILETIME, cItem.dwCount);

	memset(cItem.pvDataValues, 0, sizeof(VARIANT)*cItem.dwCount);
	memset(cItem.pdwQualities, 0, sizeof(DWORD)*cItem.dwCount);
	memset(cItem.pftTimeStamps, 0, sizeof(FILETIME)*cItem.dwCount);

	// fill in values.
	for (UINT ii = 0; ii < cItem.dwCount; ii++)
	{
		UINT uIndex = pIndicies[ii + uStartIndex];

		// handle case where bounding values are outside the range of valid values.
		if (uIndex == -1)
		{
			cItem.pdwQualities[ii] = OPCHDA_NOBOUND | OPC_QUALITY_BAD;

			if (ii == 0)
			{
				cItem.pftTimeStamps[ii] = OpcHdaFILETIMEFromInt64(llStartTime);
			}
			else if (ii == uCount-1)
			{				
				cItem.pftTimeStamps[ii] = OpcHdaFILETIMEFromInt64(llEndTime);
			}
		}

		// fetch the value at the index.
		else
		{
			cItem.pftTimeStamps[ii] = OpcHdaFILETIMEFromInt64(m_cValues[uIndex].llTimestamp);
			cItem.pvDataValues[ii]  = GetValueIndex(uIndex, cItem.pdwQualities[ii]);
		}
	}

	// free memory.
	OpcFree(pIndicies);

	// return result.
	return (bNoMoreData)?S_OK:OPC_S_MOREDATA;
}

VARIANT COpcHdaItemValues::GetValueIndex(UINT uIndex, DWORD& dwQuality)
{
	// check for invalid index.
	if (!m_cValues.IsValidIndex(uIndex))
	{
		VARIANT vValue;
		VariantInit(&vValue);

		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return vValue;
	}

	dwQuality = m_cValues[uIndex].dwQuality;
	return GetValueVariant(m_cValues[uIndex].value, dwQuality);
}

VARIANT COpcHdaItemValues::GetValueVariant(COpcVariant & value, DWORD& dwQuality)
{
	VARIANT vValue;
	VariantInit(&vValue);

//	HRESULT hResult = COpcVariant::ChangeType(vValue, value.GetRef(), NULL, item->m_vtDataType);
	HRESULT hResult = COpcVariant::ChangeType(vValue, value.GetRef(), NULL, value.GetType());

	if (FAILED(hResult))
	{
		VariantClear(&vValue);
		dwQuality = OPCHDA_CONVERSION | OPC_QUALITY_BAD;
	}

	return vValue;
}


VARIANT COpcHdaItemValues::GetValueDouble(double value, DWORD& dwQuality)
{
	COpcVariant cVariant(value);

	VARIANT vValue;
	VariantInit(&vValue);

//	HRESULT hResult = COpcVariant::ChangeType(vValue, cVariant.GetRef(), NULL, item->m_vtDataType);
	HRESULT hResult = COpcVariant::ChangeType(vValue, cVariant.GetRef(), NULL, VT_R8);

	if (FAILED(hResult))
	{
		VariantClear(&vValue);
		dwQuality = OPCHDA_CONVERSION | OPC_QUALITY_BAD;
	}

	return vValue;
}

VARIANT COpcHdaItemValues::GetValueAtTime(LONGLONG time, DWORD& dwQuality){
	UINT uLower = 0;
	UINT uUpper = 0;

	// look of the item values that bound this value.
	if (m_cValues.FindBounds(time, uLower, uUpper))
	{
		if (IsUseableData(uLower))
		{
			dwQuality = m_cValues[uLower].dwQuality;
			return m_cValues[uLower].value;
		}
	}

	bool bValuesSkipped = false;

	// search for good quality lower bound.
	while (uLower != -1 && !IsUseableData(uLower))
	{
		// no useable data found prior to the  start range.
		if (uLower == 0)
		{
			uLower = -1;
			break;
		}

		uLower--;
		bValuesSkipped = true;
	}

	return GetValueIndex(uLower, dwQuality);
}


// IsUseableData
bool COpcHdaItemValues::IsUseableData(UINT uIndex)
{
	if (m_cValues.IsValidIndex(uIndex))
	{
		return IsUseableData(m_cValues[uIndex]);
	}

	return false;
}

// IsUseableData
bool COpcHdaItemValues::IsUseableData(const COpcHdaItemValue& cValue)
{
	return OPC_MASK_ISSET(cValue.dwQuality, OPC_QUALITY_GOOD);

	return false;
}

void COpcHdaItemValues::FixProcessing(LONGLONG a,LONGLONG b){
	LONGLONG start=a<b?a:b;
	LONGLONG end=a<b?b:a;
	
	if(end-start<=1) return;

	int count=m_cValues.GetCount();
	
	if(count>0 && m_cValues[0].llTimestamp<=start){
		COpcHdaItemValue val=m_cValues[0];
		val.llTimestamp=start+1;

		m_cValues.RemoveAt(0);
		m_cValues.Insert(start+1,val);
	}

	if(count>1 && m_cValues[count-1].llTimestamp>=end){
		COpcHdaItemValue val=m_cValues[count-1];
		val.llTimestamp=end-1;

		m_cValues.RemoveAt(count-1);
		m_cValues.Insert(end-1,val);
	}

	FixBoundingTimes();
}

	
	// FixBoundingTimes
void COpcHdaItemValues::FixBoundingTimes()
{	
	m_llStartTime   = 0;
	m_llEndTime     = 0;
	m_uFirstUseable = -1;
	m_uLastUseable  = -1;

	if (m_cValues.GetCount() > 0)
	{
		m_llStartTime = m_cValues[0].llTimestamp;
		m_llEndTime   = m_cValues.GetLastValue().llTimestamp;

		// determine first useable value.
		for (UINT ii = 0; ii < m_cValues.GetCount(); ii++)
		{
			if (IsUseableData(m_cValues[ii]))
			{
				m_uFirstUseable = ii;
				break;
			}
		}

		// determine last useable value.
		for (ii = m_cValues.GetCount()-1; ii >= m_uFirstUseable; ii--)
		{
			if (IsUseableData(m_cValues[ii]))
			{
				m_uLastUseable = ii;
				break;
			}
		}
	}
}

// Interpolate
double COpcHdaItemValues::Interpolate(LONGLONG llTime, DWORD& dwQuality)
{
	UINT uLower = 0;
	UINT uUpper = 0;

	// look of the item values that bound this value.
	if (m_cValues.FindBounds(llTime, uLower, uUpper))
	{
		if (IsUseableData(uLower))
		{
			dwQuality = m_cValues[uLower].dwQuality;
			return m_cValues[uLower].value;
		}
	}

	bool bValuesSkipped = false;

	// search for good quality lower bound.
	while (uLower != -1 && !IsUseableData(uLower))
	{
		// no useable data found prior to the  start range.
		if (uLower == 0)
		{
			uLower = -1;
			break;
		}

		uLower--;
		bValuesSkipped = true;
	}

	// cannot interpolate if there is no beginning bound.
	if (uLower == -1)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// search for good quality upper bound.
	while (uUpper != -1 && !IsUseableData(uUpper))
	{
		// no useable data found prior to the  start range.
		if (uUpper == m_cValues.GetCount()-1)
		{
			uUpper = -1;
			break;
		}

		uUpper++;
		bValuesSkipped = true;
	}

	// extrapolate the last good value if there is no end bound.
	if (uUpper == -1)
	{	
		dwQuality = OPCHDA_INTERPOLATED | OPC_QUALITY_SUB_NORMAL;
		return m_cValues[uLower].value;
	}

	// calculate interpolated value.
	COpcHdaItemValue& cLower = m_cValues[uLower];
	COpcHdaItemValue& cUpper = m_cValues[uUpper];

	// set quality from the quality of the values used in the calculation.
	dwQuality = OPCHDA_INTERPOLATED;

	if (OPC_MASK_ISSET(cLower.dwQuality, OPC_QUALITY_GOOD) && OPC_MASK_ISSET(cUpper.dwQuality, OPC_QUALITY_GOOD))
	{
		dwQuality |= OPC_QUALITY_GOOD;
	}
	else
	{
		dwQuality |= OPC_QUALITY_SUB_NORMAL;
	}

	double dblValue = 0;

	// use linear interpolation.
	if (!m_bStepped)
	{
		double dx = (cUpper.asDouble() - cLower.asDouble());
		double dt = (double)(cUpper.llTimestamp - cLower.llTimestamp);
		double t  = (double)(llTime - cLower.llTimestamp);

		dblValue = ((dx*t)/dt) + cLower.asDouble();
	}

	// use stepped interpolation.
	else
	{
		dblValue = cLower.asDouble();
	}

	// override the quality if values had to be skipped to find the bounds.
	if (bValuesSkipped)
	{
		dwQuality = OPCHDA_INTERPOLATED | OPC_QUALITY_SUB_NORMAL;
	}

	// return value.
	return dblValue;
}

// TimeAverage
double COpcHdaItemValues::TimeAverage(LONGLONG llStartBound, LONGLONG llEndBound, DWORD& dwQuality)
{	
	// swap the start and end times if end is earlier than start.
	if (llStartBound > llEndBound)
	{
		LONGLONG llTime = llEndBound;
		llEndBound = llStartBound;
		llStartBound = llTime;
	}

	// use interpolation to find the value at the start.
	double dblStart = Interpolate(llStartBound, dwQuality);

	// no start an the start but it is still possible to calculate a partial aggregate.
	bool bNoStartValue = false;
	bool bPartial      = false;

	if (OPC_MASK_ISSET(dwQuality, OPCHDA_NODATA))
	{
		dwQuality = OPCHDA_PARTIAL | OPC_QUALITY_SUB_NORMAL;
		bNoStartValue = true;
		bPartial = true;
	}

	// preserve the data quality bits for the start value.
	else 
	{
		dwQuality = (OPCHDA_CALCULATED | (0x0000FFFF & dwQuality));
	}

	// get index of first data point in interval.
	UINT uIndex = m_cValues.FindIndexAfter(llStartBound);

	if (uIndex == -1)
	{
		// no data if no start bound either.
		if (bNoStartValue)
		{
			dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
			return 0;
		}

		// entire interval is after the last data point - extrapolate the start value.
		dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_SUB_NORMAL;
		return dblStart;
	}

	// the interval is calculated from the first valid data point.
	double dblInterval = ((double)(llEndBound - llStartBound))/TICKS_PER_SECOND;

	// find all useable raw data points in the the interval.
	bool bValuesSkipped = false;		

	LONGLONG llLastTime = llStartBound;
	double dblLastValue = dblStart;
		
	double dblTotal = 0;

	while (m_cValues.IsValidIndex(uIndex) && m_cValues[uIndex].llTimestamp < llEndBound)
	{
		if (!IsUseableData(uIndex))
		{
			uIndex++;
			bValuesSkipped = true;
			continue;
		}

		// calculate the time average as soon as a valid start value is found.
		if (!bNoStartValue)
		{
			double dblSpan = ((double)(m_cValues[uIndex].llTimestamp - llLastTime))/TICKS_PER_SECOND;
			double dblRise = m_cValues[uIndex].asDouble() - dblLastValue;

			if (dblSpan > 0)
			{
				dblTotal += dblRise*dblSpan/2 + dblLastValue*dblSpan;
			}
		}

		// shorten the interval so it starts from the first good value.
		else
		{
			dblInterval = ((double)(llEndBound - m_cValues[uIndex].llTimestamp))/TICKS_PER_SECOND;
		}

		// uses the current value as the base value for the next interval.
		llLastTime   = m_cValues[uIndex].llTimestamp;
		dblLastValue = m_cValues[uIndex].asDouble();
		bNoStartValue = false;

		uIndex++;
	}

	// no start value and no good data in the interval.
	if (bNoStartValue)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// went past the end of the data - extrapolate the last useable data.
	if (!m_cValues.IsValidIndex(uIndex))
	{
		// add final area.
		dblTotal += dblLastValue*(((double)(llEndBound - llLastTime))/TICKS_PER_SECOND);
		
		// set quality.
		dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_SUB_NORMAL;
	}
	
	// use the intepolated end value to calculate the final area.
	else
	{
		DWORD dwEndQuality = 0;
		double dbEndValue = Interpolate(llEndBound, dwEndQuality);

		// check if good quality is overridden by an uncertain quality at the end point.
		if (!OPC_MASK_ISSET(dwQuality, OPC_QUALITY_GOOD) || !OPC_MASK_ISSET(dwEndQuality, OPC_QUALITY_GOOD))
		{
			dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_SUB_NORMAL;
		}

		// add final area.
		double dblSpan = ((double)(llEndBound - llLastTime))/TICKS_PER_SECOND;
		double dblRise = dbEndValue - dblLastValue;

		if (dblSpan > 0)
		{
			dblTotal += dblRise*dblSpan/2 + dblLastValue*dblSpan;
		}
	}

	// divide area under curve by interval. 
	dblTotal /= dblInterval;
	
	// set partial quality bit.
	if (bPartial && OPC_MASK_ISSET(dwQuality, OPCHDA_CALCULATED))
	{
		dwQuality = OPCHDA_PARTIAL | (0x0000FFFF & dwQuality);
	}

	return dblTotal;
}

// Total
double COpcHdaItemValues::Total(LONGLONG llStartBound, LONGLONG llEndBound, DWORD& dwQuality)
{
	// calculate the time average.
	double dblValue = TimeAverage(llStartBound, llEndBound, dwQuality);

	double dblInterval = ((double)(llEndBound - llStartBound))/TICKS_PER_SECOND;

	// must determine the actual interval.
	if (OPC_MASK_ISSET(dwQuality, OPCHDA_PARTIAL))
	{
		// swap bounds if start time is larger than end time.
		if (llStartBound > llEndBound)
		{
			LONGLONG llTimestamp = llStartBound;
			llStartBound = llEndBound;
			llEndBound = llTimestamp;
		}

		// find the first good value.
		UINT uIndex = m_cValues.FindIndexAfter(llStartBound);

		while (m_cValues.IsValidIndex(uIndex) && m_cValues[uIndex].llTimestamp < llEndBound)
		{		
			if (IsUseableData(uIndex))
			{				
				dblInterval = ((double)(llEndBound - m_cValues[uIndex].llTimestamp))/TICKS_PER_SECOND;
				break;
			}
			
			uIndex++;
		}
	}
	
	// multiple by the interval in seconds.
	dblValue *= dblInterval;
	
	// return result.
	return dblValue;
}

// FindRawValues
bool COpcHdaItemValues::FindRawValues(LONGLONG llStartTime, LONGLONG llEndTime, COpcList<COpcHdaItemValue*>& cValues)
{
	// determine if series is ascending or descending.
	bool bAscending = (llStartTime < llEndTime);

	// get lower and upper bounding times for the series.
	LONGLONG llLower = (bAscending)?llStartTime:llEndTime;
	LONGLONG llUpper = (bAscending)?llEndTime:llStartTime;

	// find indicies.
	UINT uLower = m_cValues.FindIndexAfter(llLower);
	UINT uUpper = m_cValues.FindIndexBefore(llUpper);

	// exclude values that are on the end bound.
	if (bAscending)
	{
		while (m_cValues.IsValidIndex(uUpper) && m_cValues[uUpper].llTimestamp >= llUpper)
		{
			uUpper--;
		}
	}
	else
	{
		while (m_cValues.IsValidIndex(uLower) && m_cValues[uLower].llTimestamp <= llLower)
		{
			uLower++;
		}
	}

	// check for data within range.
	if (uLower == -1 || uUpper == -1 || uLower > uUpper)
	{
		return false;
	}

	// add values all values within interval.
	for (UINT ii = uLower; ii <= uUpper; ii++)
	{
		cValues.AddTail(&m_cValues[ii]);
	}

	// all done.
	return true;
}

// FindUsableValues
DWORD COpcHdaItemValues::FindUseableValues(LONGLONG llStartTime, LONGLONG llEndTime, COpcList<COpcHdaItemValue*>& cValues)
{
	// find raw values in interval.
	COpcList<COpcHdaItemValue*> cRawValues;

	if (!FindRawValues(llStartTime, llEndTime, cRawValues))
	{
		return OPCHDA_NODATA | OPC_QUALITY_BAD;
	}

	// initialize quality.
	DWORD dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_GOOD;

	// remove bad values.
	while (cRawValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cRawValues.RemoveHead();

		if (!IsUseableData(*pValue))
		{
			dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_SUB_NORMAL;
			continue;
		}

		cValues.AddTail(pValue);
	}

	// no usable data found.
	if (cValues.GetCount() == 0)
	{
		return OPCHDA_NODATA | OPC_QUALITY_BAD;
	}

	// check for partial intervals.
	if (m_uFirstUseable!=-1 && llStartTime < m_cValues[m_uFirstUseable].llTimestamp)
	{
		dwQuality = OPCHDA_PARTIAL | (0x0000FFFF & dwQuality);
	}

	// all done.
	return dwQuality;
}

// Average
double COpcHdaItemValues::Average(LONGLONG llStartTime, LONGLONG llEndTime, DWORD& dwQuality)
{
	// find usable values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	dwQuality = FindUseableValues(llStartTime, llEndTime, cValues);

	if (OPC_MASK_ISSET(dwQuality, OPCHDA_NODATA))
	{
		return 0;
	}

	// calculate sum and count.
	double dblSum   = 0;
	double dblCount = 0;
	
	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();

		dblSum += pValue->asDouble();
		dblCount++;
	}

	// should never happen.
	if (dblCount == 0)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// calculate average.
	return dblSum/dblCount;
}

// Count
double COpcHdaItemValues::Count(LONGLONG llStartTime, LONGLONG llEndTime, DWORD& dwQuality)
{
	// find raw values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	if (!FindRawValues(llStartTime, llEndTime, cValues))
	{
		dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_GOOD;
		return 0;
	}

	// initialize quality.
	dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_GOOD;

	// count the number of good values.
	double dblCount = 0;
	
	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();

		if (!IsUseableData(*pValue))
		{
			dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_SUB_NORMAL;
			continue;
		}

		dblCount++;
	}

	// calculate average.
	return dblCount;
}

// StdDev
double COpcHdaItemValues::StdDev(bool bVariance, LONGLONG llStartTime, LONGLONG llEndTime, DWORD& dwQuality)
{
	// find usable values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	dwQuality = FindUseableValues(llStartTime, llEndTime, cValues);

	if (OPC_MASK_ISSET(dwQuality, OPCHDA_NODATA))
	{
		return 0;
	}

	// should never happen.
	if (cValues.GetCount() == 0)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// calculate sum and count.
	double dblSum   = 0;
	double dblCount = 0;

	OPC_POS pos = cValues.GetHeadPosition();

	while (pos != NULL)
	{
		COpcHdaItemValue* pValue = cValues.GetNext(pos);
		
		dblSum += pValue->asDouble();
		dblCount++;
	}

	// check for trivial case.
	if (dblCount == 1)
	{
		dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_GOOD;
		return 0;
	}

	double dblAverage = dblSum/dblCount;
	
	dblSum = 0;

	// calculate sum of differences
	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();

		dblSum += pow((pValue->asDouble()-dblAverage), 2);
	}

	// return the variance.
	if (bVariance)
	{
		return dblSum/(dblCount-1);
	}

	// calculate standard deviation.
	return sqrt(dblSum/(dblCount-1));
}

// MinMax
double COpcHdaItemValues::MinMax(
    bool      bMinimum,
	LONGLONG  llStartTime, 
	LONGLONG  llEndTime, 
	DWORD&    dwQuality,
	FILETIME* pftTimestamp
)
{
	// find usable values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	dwQuality = FindRawValues(llStartTime, llEndTime, cValues);

	if (OPC_MASK_ISSET(dwQuality, OPCHDA_NODATA))
	{
		return 0;
	}

	// should never happen.
	if (cValues.GetCount() == 0)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// initialize quality.
	dwQuality = OPCHDA_RAW | OPC_QUALITY_GOOD;

	// find minimum value.

	COpcHdaItemValue* pGood = NULL;
	COpcHdaItemValue* pBad  = NULL;

	bool bExtraData = false;

	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();

		// check for bad data.
		if (!IsUseableData(*pValue))
		{
			// no data values cannot be used in a comparison.
			if (!OPC_MASK_ISSET(pValue->dwQuality, OPCHDA_NODATA))
			{
				if ((pBad == NULL) || (bMinimum && pValue->asDouble() < pBad->asDouble()) || (!bMinimum && pValue->asDouble() > pBad->asDouble()))
				{
					pBad = pValue;
				}
			}

			continue;
		}

		// check if the current value is a new minimum/maximum.
		if ((pGood == NULL) || (bMinimum && pValue->asDouble() < pGood->asDouble()) || (!bMinimum && pValue->asDouble() > pGood->asDouble()))
		{
			pGood      = pValue;
			bExtraData = false;
		}

		// check if the current value is equal to the minimum/maximum.
		else if (pValue->asDouble() == pGood->asDouble())
		{
			bExtraData  = true;
		}
	}
	
	// check if no good data exists - use bad value instead.
	if (pGood == NULL)
	{
		pGood = pBad;
	
	}
	// should never happen.
	if (pGood == NULL)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// initialize quality.
	dwQuality = pGood->dwQuality;

	// check if a bad value exists and exceeded the good value.
	if (pBad != NULL)
	{
		if ((bMinimum && pBad->asDouble() < pGood->asDouble()) || (!bMinimum && pBad->asDouble() > pGood->asDouble()))
		{
			dwQuality = OPCHDA_RAW | OPC_QUALITY_SUB_NORMAL;
		}
	}

	// return actual time if required.
	if (pftTimestamp != NULL)
	{
		if (bExtraData)
		{
			dwQuality = (OPCHDA_EXTRADATA | (0x0000FFFF & dwQuality));
		}

		*pftTimestamp = OpcHdaFILETIMEFromInt64(pGood->llTimestamp);
	}

	// use start of interval.
	else
	{
		if (pGood->llTimestamp == llStartTime)
		{
			dwQuality = (OPCHDA_RAW | (0x0000FFFF & dwQuality));
		}
		else
		{
			dwQuality = (OPCHDA_CALCULATED | (0x0000FFFF & dwQuality));
		}
	}

	// return minimum value.
	return pGood->asDouble();
}

// StartEnd
double COpcHdaItemValues::StartEnd(
    bool      bStart,
	LONGLONG  llStartTime, 
	LONGLONG  llEndTime, 
	DWORD&    dwQuality,
	FILETIME& ftTimestamp
)
{
	// find usable values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	dwQuality = FindRawValues(llStartTime, llEndTime, cValues);

	if (OPC_MASK_ISSET(dwQuality, OPCHDA_NODATA))
	{
		return 0;
	}

	// get the start or end raw value.
	COpcHdaItemValue* pValue = NULL;
	
	while (cValues.GetCount() > 0 && pValue == NULL)
	{
		pValue = (bStart)?cValues.RemoveHead():cValues.RemoveTail();

		if (OPC_MASK_ISSET(pValue->dwQuality, OPCHDA_NODATA))
		{
			pValue = NULL;
		}
	}

	// no data found in the interval.
	if (pValue == NULL)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// set raw quality and timestamp.
	dwQuality   = pValue->dwQuality;
	ftTimestamp = OpcHdaFILETIMEFromInt64(pValue->llTimestamp);

	// return value.
	return pValue->asDouble();
}

// Regression
double COpcHdaItemValues::Regression(bool bSlope, LONGLONG llStartTime, LONGLONG llEndTime, DWORD& dwQuality)
{
	// calculate linear regression.
	double dblSlope    = 0;
	double dblConstant = 0;

	dwQuality = LinearRegression(llStartTime, llEndTime, dblSlope, dblConstant);

	// return requested value.
	return (bSlope)?dblSlope:dblConstant;
}

// RegressionDeviation
double COpcHdaItemValues::RegressionDeviation(LONGLONG llStartTime, LONGLONG llEndTime, DWORD& dwQuality)
{
	// calculate linear regression.
	double A = 0;
	double B = 0;

	// determine if series is ascending or descending.
	bool bAscending = ((llStartTime != 0 && llStartTime <= llEndTime) || llEndTime == 0);

	// get lower and upper bounding times for the series.
	LONGLONG llLower = (bAscending)?llStartTime:llEndTime;
	LONGLONG llUpper = (bAscending)?llEndTime:llStartTime;

	dwQuality = LinearRegression(llStartTime, llEndTime, A, B);

	// check for set of unchanging points - this implies a zero error term.
	if (A == 0)
	{
		return 0;
	}	
	
	// find usable values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	dwQuality = FindUseableValues(llStartTime, llEndTime, cValues);

	if (cValues.GetCount() == 0)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// calculate the total error term.
	double dblError = 0;
	double dblCount = 0;

	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();
		
		// get next point.
		double x = ((double)(pValue->llTimestamp - llLower))/TICKS_PER_SECOND;
		double y = pValue->asDouble();

		// add to sums.
		dblError += pow((y - (A*x + B)), 2);
		dblCount += 1;
	}

	// can be no error for regressions calculted with 2 or fewer points.
	if (dblCount < 3)
	{
		return 0;
	}

	// return square root of the error term.
	return sqrt(dblError/(dblCount-2));
}

// LinearRegression
DWORD COpcHdaItemValues::LinearRegression(
	LONGLONG llStartTime, 
	LONGLONG llEndTime, 
	double&  dblSlope,
	double&  dblConstant)
{	
	dblSlope    = 0;
	dblConstant = 0;

	// determine if series is ascending or descending.
	bool bAscending = ((llStartTime != 0 && llStartTime <= llEndTime) || llEndTime == 0);

	// get lower and upper bounding times for the series.
	LONGLONG llLower = (bAscending)?llStartTime:llEndTime;
	LONGLONG llUpper = (bAscending)?llEndTime:llStartTime;

	// find usable values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	DWORD dwQuality = FindUseableValues(llStartTime, llEndTime, cValues);

	if (cValues.GetCount() == 0)
	{
		return OPCHDA_NODATA | OPC_QUALITY_BAD;
	}
	
	// handle the special case of one point.
	if (cValues.GetCount() == 1)
	{
		return OPCHDA_CALCULATED | OPC_QUALITY_GOOD;
	}

	//==========================================
	// Linear regression uses the formula:
	//
	// Slope    = (XY*N -  X*Y)/(XX*N - X*X)
	// Constant = (XX*Y - XY*X)/(XX*N - X*X)
	//
	// Where X  = Sum of Timestamps;
	//       Y  = Sum of Values;
	//       XY = Sum of Timestamp*Value;
	//       XX = Sum of Timestamp*Timestamp;
	//       N  = Number of Values;
	//==========================================

	double X  = 0;
	double Y  = 0;
	double XY = 0;
	double XX = 0;
	double N  = 0;

	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();
		
		// get next point.
		double x = ((double)(pValue->llTimestamp-llLower))/TICKS_PER_SECOND;
		double y = pValue->asDouble();

		// add to sums.
		X  += x;
		Y  += y;
		XX += x*x;
		XY += x*y;
		N  += 1;
	}

	// check if regression can be calculated.
	double dblDivisor = (XX*N - X*X);

	if (dblDivisor == 0)
	{
		dblSlope    = 0;
		dblConstant = (double)0xFFF8000000000000;
		return dwQuality;
	}

	// calculate the slope and constant.
	dblSlope    = (XY*N -  X*Y)/(XX*N - X*X);
	dblConstant = (XX*Y - XY*X)/(XX*N - X*X);

	// return quality.
	return dwQuality;
}

// Delta
double COpcHdaItemValues::Delta(LONGLONG llStartTime, LONGLONG llEndTime, DWORD& dwQuality)
{	
	// find raw values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	dwQuality = FindRawValues(llStartTime, llEndTime, cValues);

	if (cValues.GetCount() == 0)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	bool bValuesSkipped = false;

	// find the start value.
	COpcHdaItemValue* pHead = NULL;

	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();

		if (IsUseableData(*pValue))
		{
			pHead = pValue;
			break;
		}

		bValuesSkipped = true;
	}

	// find the last value.
	COpcHdaItemValue* pTail = pHead;

	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveTail();

		if (IsUseableData(*pValue))
		{
			pTail = pValue;
			break;
		}

		bValuesSkipped = true;
	}

	// no good data found.
	if (pHead == NULL || pTail == NULL)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// initialize the quality.
	dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_GOOD;

	if (bValuesSkipped)
	{
		dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_SUB_NORMAL;
	}

	// check for partial intervals.
	if (m_uFirstUseable!=-1 && llStartTime < m_cValues[m_uFirstUseable].llTimestamp)
	{
		dwQuality = OPCHDA_PARTIAL | (0x0000FFFF & dwQuality);
	}

	// return the difference between the end and the start.
	return (pTail->asDouble() - pHead->asDouble());
}

// Range
double COpcHdaItemValues::Range(LONGLONG llStartTime, LONGLONG llEndTime, DWORD& dwQuality)
{	
	// find usable values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	dwQuality = FindUseableValues(llStartTime, llEndTime, cValues);

	if (cValues.GetCount() == 0)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// find the min and max.
	COpcHdaItemValue* pMin = NULL;
	COpcHdaItemValue* pMax = NULL;

	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();

		// check for new minimum.
		if (pMin == NULL || pMin->asDouble() > pValue->asDouble())
		{
			pMin = pValue;
		}
				
		// check for new maximum.
		if (pMax == NULL || pMax->asDouble() < pValue->asDouble())
		{
			pMax = pValue;
		}
	}

	// check for valid data.
	if (pMin == NULL || pMax == NULL)
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	// return the difference between the minium and the maximum.
	return (pMax->asDouble() - pMin->asDouble());
}

// calculates the time or percentage of an interval that is either good/bad.
double COpcHdaItemValues::Duration(
	bool      bGood,
	bool      bPercent,
	LONGLONG  llStartTime, 
	LONGLONG  llEndTime, 
	DWORD&    dwQuality
)
{
	double dblGood = 0;
	double dblBad  = 0;

	// initialize quality.
	dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_GOOD;

	// determine if series is ascending or descending.
	bool bAscending = ((llStartTime != 0 && llStartTime <= llEndTime) || llEndTime == 0);

	// get lower and upper bounding times for the series.
	LONGLONG llLower = (bAscending)?llStartTime:llEndTime;
	LONGLONG llUpper = (bAscending)?llEndTime:llStartTime;

	// find indicies.
	UINT uLower = m_cValues.FindIndexAfter(llLower);
	UINT uUpper = m_cValues.FindIndexBefore(llUpper);

	// check for range out of bounds.
	if (uLower == -1 || uUpper == -1)
	{
		if ((bAscending && uUpper == -1) || (!bAscending && uLower == -1))
		{
			dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
			return 0;
		}

		UINT uBound = (uUpper != -1)?uUpper:uLower;

		if (OPC_MASK_ISSET(m_cValues[uBound].dwQuality, OPC_QUALITY_GOOD))
		{
			dblGood = (double)(llUpper - llLower);
		}	

		if (OPC_MASK_ISSET(~m_cValues[uBound].dwQuality, OPC_QUALITY_GOOD))
		{
			dblBad = (double)(llUpper - llLower);
		}
	}

	// calculate durations.
	else if (bAscending)
	{		
		// find bounding value at the start of the interval.
		DWORD    dwLastQuality   = OPC_QUALITY_GOOD;
		LONGLONG llLastTimestamp = llStartTime;

		if (m_cValues[uLower].llTimestamp > llStartTime)
		{
			dwLastQuality = (uLower > 0)?m_cValues[uLower-1].dwQuality:OPC_QUALITY_BAD;
		}

		// move through each point in the interval.
		for (UINT ii = uLower; ii <= uUpper; ii++)
		{
			// check if range is good.
			if (OPC_MASK_ISSET(dwLastQuality, OPC_QUALITY_GOOD))
			{
				dblGood += (double)(m_cValues[ii].llTimestamp - llLastTimestamp);
			}

			// check if range is bad.
			if (OPC_MASK_ISSET(~dwLastQuality, OPC_QUALITY_GOOD))
			{
				dblBad += (double)(m_cValues[ii].llTimestamp - llLastTimestamp);
			}

			dwLastQuality   = m_cValues[ii].dwQuality; 
			llLastTimestamp = m_cValues[ii].llTimestamp;
		}

		// add remainder at end.
		if (llLastTimestamp < llEndTime)
		{
			// check if range is good.
			if (OPC_MASK_ISSET(dwLastQuality, OPC_QUALITY_GOOD))
			{
				dblGood += (double)(llEndTime - llLastTimestamp);
			}

			// check if range is bad.
			if (OPC_MASK_ISSET(~dwLastQuality, OPC_QUALITY_GOOD))
			{
				dblBad += (double)(llEndTime - llLastTimestamp);
			}
		}
	}
	else
	{
		// find bounding value at the start of the interval.
		DWORD    dwLastQuality   = OPC_QUALITY_GOOD;
		LONGLONG llLastTimestamp = llStartTime;

		if (m_cValues[uUpper].llTimestamp < llStartTime)
		{
			dwLastQuality = (uUpper < m_cValues.GetCount()-1)?m_cValues[uUpper+1].dwQuality:OPC_QUALITY_BAD;
		}

		// move through each point in the interval.
		for (UINT ii = uUpper; ii >= uLower && ii != -1; ii--)
		{
			// check if range is good.
			if (OPC_MASK_ISSET(dwLastQuality, OPC_QUALITY_GOOD))
			{
				dblGood += (double)(llLastTimestamp - m_cValues[ii].llTimestamp);
			}

			// check if range is bad.
			if (OPC_MASK_ISSET(~dwLastQuality, OPC_QUALITY_GOOD))
			{
				dblBad += (double)(llLastTimestamp - m_cValues[ii].llTimestamp);
			}

			dwLastQuality   = m_cValues[ii].dwQuality; 
			llLastTimestamp = m_cValues[ii].llTimestamp;
		}

		// add remainder at end.
		if (llLastTimestamp > llEndTime)
		{
			// check if range is good.
			if (OPC_MASK_ISSET(dwLastQuality, OPC_QUALITY_GOOD))
			{
				dblGood += (double)(llLastTimestamp - llEndTime);
			}

			// check if range is bad.
			if (OPC_MASK_ISSET(~dwLastQuality, OPC_QUALITY_GOOD))
			{
				dblBad += (double)(llLastTimestamp - llEndTime);
			}
		}
	}

	// calculate requested values.
	if (bPercent)
	{
		dblGood /= (double)(llUpper - llLower);
		dblBad  /= (double)(llUpper - llLower);
	}
	else
	{
		dblGood /= (double)TICKS_PER_SECOND;
		dblBad  /= (double)TICKS_PER_SECOND;
	}

	// return requested value.
	return (bGood)?dblGood:dblBad;
}

// WorstQuality
double COpcHdaItemValues::WorstQuality(LONGLONG llStartTime, LONGLONG llEndTime, DWORD& dwQuality)
{
	// initialize quality.
	dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_GOOD;

	// find raw values in interval.
	COpcList<COpcHdaItemValue*> cValues;

	if (!FindRawValues(llStartTime, llEndTime, cValues))
	{
		dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;
		return 0;
	}

	WORD wWorstQuality = OPC_QUALITY_GOOD;

	while (cValues.GetCount() > 0)
	{
		COpcHdaItemValue* pValue = cValues.RemoveHead();

		// check if quality is bad.
		if (OPC_MASK_ISSET(~pValue->dwQuality, OPC_QUALITY_GOOD))
		{
			return OPC_QUALITY_BAD;
		}

		// check if quality is uncertain.
		if (!OPC_MASK_ISSET(pValue->dwQuality, OPC_QUALITY_GOOD))
		{
			wWorstQuality = OPC_QUALITY_UNCERTAIN;
		}
	}

	// return result.
	return wWorstQuality;
}

// ReadProcessed
HRESULT COpcHdaItemValues::ReadProcessed(
	LONGLONG     llStartTime,
	LONGLONG     llEndTime,
	LONGLONG     llResampleInterval,
	DWORD        haAggregate,
	OPCHDA_ITEM& cItem,
	bool         useEndBound
)
{
	LONGLONG llDelta = (llEndTime < llStartTime)?-llResampleInterval:+llResampleInterval;

	LONGLONG llStartBound = llStartTime;
	LONGLONG llEndBound   = llStartTime + llDelta;

	UINT uNumValues = 1;

	// a resample interval of zero is a special case.
	if (llResampleInterval == 0)
	{
		llEndBound = llEndTime;
		uNumValues = 1;
	}

	// calculate number of values.
	else
	{
		LONGLONG llUpperBound = (llEndTime > llStartTime)?llEndTime:llStartTime;
		LONGLONG llLowerBound = (llEndTime < llStartTime)?llEndTime:llStartTime;

		uNumValues = (UINT)(((double)(llUpperBound - llLowerBound))/llResampleInterval);

		// add a partial interval at the if range is not evenly divisible by the interval.
		if ((llUpperBound - llLowerBound)%llResampleInterval != 0)
		{
			uNumValues++;
		}
	}

	// initialize item object.
	cItem.dwCount     = uNumValues;
	cItem.haAggregate = haAggregate;
	cItem.hClient     = NULL;

	// allocate arrays.
	cItem.pvDataValues  = OpcArrayAlloc(VARIANT, uNumValues);
	cItem.pdwQualities  = OpcArrayAlloc(DWORD, uNumValues);
	cItem.pftTimeStamps = OpcArrayAlloc(FILETIME, uNumValues);

	memset(cItem.pvDataValues, 0, sizeof(VARIANT)*uNumValues);
	memset(cItem.pdwQualities, 0, sizeof(DWORD)*uNumValues);
	memset(cItem.pftTimeStamps, 0, sizeof(FILETIME)*uNumValues);

	// calculate values.
	HRESULT hResult = S_OK;

	for (UINT ii = 0; ii < uNumValues; ii++)
	{
		// check for partial aggregate.
		bool bPartial = false;

		if ((llEndTime > llStartTime && llEndBound > llEndTime) || (llEndTime < llStartTime && llEndBound < llEndTime))
		{
			llEndBound = llEndTime;
			bPartial   = true;
		}

		// default timestamp is the beginning of the interval. 
		if(useEndBound)
			cItem.pftTimeStamps[ii] = OpcHdaFILETIMEFromInt64(llEndBound);
		else
			cItem.pftTimeStamps[ii] = OpcHdaFILETIMEFromInt64(llStartBound);

		double dblValue  = 0;
		DWORD  dwQuality = OPCHDA_NODATA | OPC_QUALITY_BAD;

		if(haAggregate!=OPCHDA_COUNT){
			FixProcessing(llStartTime,llEndTime);
		}

		// calculate aggregate for the interval.
		switch (haAggregate)
		{
			case OPCHDA_INTERPOLATIVE:
			{ 
				dblValue = Interpolate(llStartBound, dwQuality);
				break;
			}

			case OPCHDA_TIMEAVERAGE:
			{ 
				dblValue = TimeAverage(llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_TOTAL:
			{ 
				dblValue = Total(llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_TOTALPLUSPLUS:
			case OPCHDA_AVERAGE:
			{ 
				dblValue = Average(llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_COUNT:
			{
				dblValue = Count(llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_STDEV:
			{ 
				dblValue = StdDev(false, llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_VARIANCE:
			{ 
				dblValue = StdDev(true, llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_MINIMUM:
			{ 
				dblValue = MinMax(true, llStartBound, llEndBound, dwQuality, NULL);
				break;
			}
			
			case OPCHDA_MAXIMUM:
			{ 
				dblValue = MinMax(false, llStartBound, llEndBound, dwQuality, NULL);
				break;
			}

			case OPCHDA_MINIMUMACTUALTIME:
			{ 
				dblValue = MinMax(true, llStartBound, llEndBound, dwQuality, &cItem.pftTimeStamps[ii]);
				break;
			}
			
			case OPCHDA_MAXIMUMACTUALTIME:
			{ 
				dblValue = MinMax(false, llStartBound, llEndBound, dwQuality, &cItem.pftTimeStamps[ii]);
				break;
			}

			case OPCHDA_START:
			{ 
				dblValue = StartEnd(true, llStartBound, llEndBound, dwQuality, cItem.pftTimeStamps[ii]);
				break;
			}
			
			case OPCHDA_END:
			{ 
				dblValue = StartEnd(false, llStartBound, llEndBound, dwQuality, cItem.pftTimeStamps[ii]);
				break;
			}
			
			case OPCHDA_DELTA:
			{ 
				dblValue = Delta(llStartBound, llEndBound, dwQuality);
				break;
			}
			
			case OPCHDA_REGSLOPE:
			{ 
				dblValue = Regression(true, llStartBound, llEndBound, dwQuality);
				break;
			}
			
			case OPCHDA_REGCONST:
			{ 
				dblValue = Regression(false, llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_REGDEV:
			{ 
				dblValue = RegressionDeviation(llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_RANGE:
			{ 
				dblValue = Range(llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_DURATIONGOOD:
			{ 
				dblValue = Duration(true, false, llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_DURATIONBAD:
			{ 
				dblValue = Duration(false, false, llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_PERCENTGOOD:
			{ 
				dblValue = Duration(true, true, llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_PERCENTBAD:
			{ 
				dblValue = Duration(false, true, llStartBound, llEndBound, dwQuality);
				break;
			}

			case OPCHDA_WORSTQUALITY:
			{ 
				dblValue = WorstQuality(llStartBound, llEndBound, dwQuality);
				break;
			}

			default:
			{
				dwQuality = OPCHDA_CALCULATED | OPC_QUALITY_BAD;
				dblValue  = 0;
				break;
			}	
		}

		// always return VT_EMPTY is no data found.
		if (OPC_MASK_ISSET(dwQuality, OPCHDA_NODATA))
		{
			cItem.pvDataValues[ii].vt = VT_EMPTY;
		}

		// convert double to item's canonical data type.
		else
		{
			cItem.pvDataValues[ii] = GetValueDouble(dblValue, dwQuality);
		}

		// add partial flag to quality.
		if (bPartial)
		{
			dwQuality = OPCHDA_PARTIAL | (0x0000FFFF & dwQuality);
		}

		// set the item quality.
		cItem.pdwQualities[ii] = dwQuality;

		// move to next interval.
		llStartBound += llDelta;
		llEndBound   += llDelta;
	}

	// return result.
	return hResult;
}

// ReadAtTime
HRESULT COpcHdaItemValues::ReadAtTime(
	COpcArray<LONGLONG>& cTimestamps,
	OPCHDA_ITEM&         cItem
)
{
	// initialize empty item object.
	cItem.dwCount     = cTimestamps.GetSize();
	cItem.haAggregate = OPCHDA_NOAGGREGATE;
	cItem.hClient     = NULL;

	// allocate arrays.
	cItem.pvDataValues  = OpcArrayAlloc(VARIANT, cItem.dwCount);
	cItem.pdwQualities  = OpcArrayAlloc(DWORD, cItem.dwCount);
	cItem.pftTimeStamps = OpcArrayAlloc(FILETIME, cItem.dwCount);

	memset(cItem.pvDataValues, 0, sizeof(VARIANT)*cItem.dwCount);
	memset(cItem.pdwQualities, 0, sizeof(DWORD)*cItem.dwCount);
	memset(cItem.pftTimeStamps, 0, sizeof(FILETIME)*cItem.dwCount);

	HRESULT hResult = S_OK;

	bool bDataFound = false;

	for (UINT ii = 0; ii < cItem.dwCount; ii++)
	{
		cItem.pvDataValues[ii].vt = VT_EMPTY;

		// fetch the raw or interpolated value for the aggregate.
		double dblValue = Interpolate(cTimestamps[ii], cItem.pdwQualities[ii]);

		// set the timestamp.
		cItem.pftTimeStamps[ii] = OpcHdaFILETIMEFromInt64(cTimestamps[ii]);

		if (!OPC_MASK_ISSET(cItem.pdwQualities[ii], OPCHDA_NODATA))
		{
			cItem.pvDataValues[ii] = GetValueAtTime(cTimestamps[ii], cItem.pdwQualities[ii]);
		}

		// set the no data flag.
		if (!OPC_MASK_ISSET(cItem.pdwQualities[ii], OPCHDA_NODATA))
		{
			bDataFound = true;
		}
	}

	// return result.
	return (bDataFound)?S_OK:OPC_S_NODATA;
}


COpcHdaAttributeValues::COpcHdaAttributeValues(COpcHdaItem *it,int count){
	Init();

	item=it;
	m_cValues.SetCapacity(count);
}

// Init
void COpcHdaAttributeValues::Init(){
	
}
         
// Clear
void COpcHdaAttributeValues::Clear(){
	m_cValues.RemoveAll();

    Init();
}

void COpcHdaAttributeValues::AppendValue(COpcVariant &value,LONGLONG when, DWORD dwAttributeID){
	COpcHdaAttributeValue* pValue = new COpcHdaAttributeValue();
	pValue->dwAttributeID=dwAttributeID;
	pValue->llTimestamp=when;
	pValue->cValue=value;
	m_cValues.Append(pValue->llTimestamp, *pValue);
}
