#include "interface.h"
#include "COpcHdaHistorian.h"
#include "COpcHdaAttribute.h"


OpcHdaValue *(*GetValuesForItemFunc)(void *userObject,LONGLONG llStartTime,LONGLONG llEndTime,OpcHdaValue *(*allocator)(int count,void *object),void *object);
OpcHdaAttribute *(*GetAttributesForItemFunc)(void *userObject,LONGLONG llStartTime,LONGLONG llEndTime,OpcHdaAttribute *(*allocator)(int count,void *object),void *object);
bool processFixed;

struct GetValuesForItemAllocatorHelper{
	int count;
};

template<class t>
static t *GetValuesForItemAllocator(int count,void *object){
	GetValuesForItemAllocatorHelper *helper=(GetValuesForItemAllocatorHelper *)object;
	
	if(count<0) return NULL;
	if(helper->count!=-1) return NULL;
	
	t *res=new t[count];
	helper->count=count;
	return res;
}

COpcHdaItemValues *GetValuesForItem(COpcHdaItem *item,LONGLONG llStartTime,LONGLONG llEndTime){
	GetValuesForItemAllocatorHelper helper;
	OpcHdaValue *values=NULL;
	helper.count=-1;
	
	if(GetValuesForItemFunc!=NULL){
		values=GetValuesForItemFunc(item->getUserObject(),llStartTime,llEndTime,GetValuesForItemAllocator,&helper);
	}

	if(helper.count==-1 || values==NULL)
		helper.count=0;

	COpcHdaItemValues *res=new COpcHdaItemValues(item,helper.count);

	for(int i=0;i<helper.count;i++){
		COpcVariant var(values[i].value);
		res->AppendValue(var,values[i].timestamp,values[i].quality);
	}

	if(values!=NULL) delete [] values;

	return res;
}

COpcHdaAttributeValues *GetAttributesForItem(COpcHdaItem *item,LONGLONG llStartTime,LONGLONG llEndTime){
	GetValuesForItemAllocatorHelper helper;
	OpcHdaAttribute *values=NULL;
	helper.count=-1;
	
	if(GetAttributesForItemFunc!=NULL){
		values=GetAttributesForItemFunc(item->getUserObject(),llStartTime,llEndTime,GetValuesForItemAllocator,&helper);
	}

	if(helper.count==-1 || values==NULL)
		helper.count=0;

	COpcHdaAttributeValues *res=new COpcHdaAttributeValues(item,helper.count);

	for(int i=0;i<helper.count;i++){
		COpcVariant var(values[i].value);
		res->AppendValue(var,values[i].timestamp,values[i].attrId);
	}

	if(values!=NULL) delete [] values;

	return res;
}

#include "StdAfx.h"
#include "COpcHdaServerImpl.h"

static TOpcClassTableEntry g_pClassTable[] = {
	{
		&(__uuidof(OpcHdaServer)),
		_T("ComfortContourOPCHDA"),
		_T("1"),
		_T("OPC Historical Data Access 1.20 Server"),
		(PfnOpcCreateInstance*)COpcHdaServerImpl::CreateInstance
	}, {
		&CLSID_NULL,
		NULL,
		NULL,
		NULL,
		NULL
	},
};


static TClassCategories g_pCategoryTable[] = {
	{&(__uuidof(OpcHdaServer)), &(CATID_OPCHDAServer10), (OPC_CATEGORY_DESCRIPTION_HDA10)},
	{NULL, NULL, NULL},
};

COpcComModule *pModule;

DWORD WINAPI OpcThreadProc(LPVOID lpParam){
	DWORD dwResult = OpcWinMain(pModule, ((HINSTANCE)GetModuleHandle(NULL)), _T(""));

    delete pModule;

    return dwResult;
}

const GUID noguid = {0x778f4d15, 0x8401, 0x43d8, { 0xa8, 0x8d, 0xfb, 0xfd, 0x8, 0x35, 0x38, 0xb3 } };

static void start(bool threaded){
	if(threaded){
		CreateThread(NULL,0,OpcThreadProc,pModule,0,NULL);
	} else{
		OpcThreadProc(pModule);
	}
}

LIBOPCHDA_INTERFACE void OpcHdaInitialize(){
	if(pModule) return;
	
	pModule = new COpcComModule(
		L"UnknownVendor",
		L"UnknownApplication",
		L"Nondescript",
		noguid,
		g_pClassTable,
		g_pCategoryTable);

	pModule->RegisterFromFiles(((HINSTANCE)GetModuleHandle(NULL)));

	IUnknown *ipUnknown;
	g_pClassTable[0].pfnCreateInstance(&ipUnknown, g_pClassTable[0].pClsid);
}
LIBOPCHDA_INTERFACE void OpcHdaStart(){
	start(false);	
}
LIBOPCHDA_INTERFACE void OpcHdaStartThreaded(){
	start(true);
}

LIBOPCHDA_INTERFACE void OpcHdaRegister(){
	if(pModule==NULL) return;

	pModule->RegisterServer(false);
}

LIBOPCHDA_INTERFACE void OpcHdaUnregister(){
	if(pModule==NULL) return;

	pModule->UnregisterServer();
}

LIBOPCHDA_INTERFACE void OpcHdaRegisterValuesHandler(OpcHdaValue *(*GetValuesForItem)(void *item,LONGLONG start,LONGLONG end,OpcHdaValue *(*allocator)(int count,void *object),void *object)){
	GetValuesForItemFunc=GetValuesForItem;
}

LIBOPCHDA_INTERFACE void OpcHdaRegisterAttributesHandler(OpcHdaAttribute *(*GetAttributesForItem)(void *userObject,LONGLONG start,LONGLONG end,OpcHdaAttribute *(*allocator)(int count,void *object),void *object)){
	GetAttributesForItemFunc=GetAttributesForItem;
}

LIBOPCHDA_INTERFACE void *OpcHdaCreateItem(TCHAR *name,void *userObject){
	COpcString cItemID=name;
	COpcHdaItem* pItem = new COpcHdaItem(cItemID,userObject,&GetHistorian());
	GetHistorian().m_cItems[cItemID] = pItem;
	GetHistorian().m_cItemList.AddTail(cItemID);
	pItem->BuildAddressSpace();

	return pItem;
}

LIBOPCHDA_INTERFACE void OpcHdaDeleteItem(TCHAR *name){
	COpcString cItemID=name;

	COpcHdaItem* pItem = NULL;

	if(!GetHistorian().m_cItems.Lookup(cItemID, pItem))
		return;
	
	pItem->ClearAddressSpace();
}

LIBOPCHDA_INTERFACE void OpcHdaCreateAttribute(DWORD attrId,VARTYPE vtDataType,wchar_t *name,wchar_t *description){
	OpcHdaAttributeDesc desc;

	desc.dwID=attrId;
	desc.vtDataType=vtDataType;
	desc.name=name;
	desc.description=description;
	
	attributeTable.Append(desc);
}

LIBOPCHDA_INTERFACE void OpcHdaSetProcessedFixedMode(bool fixed){
	processFixed=fixed;
}